@@ -1,19 +1,11 @@
-## Copyright 2011-2013 Luc Saffre
-## This file is part of the Lino project.
-## Lino is free software; you can redistribute it and/or modify 
-## it under the terms of the GNU General Public License as published by
-## the Free Software Foundation; either version 3 of the License, or
-## (at your option) any later version.
-## Lino is distributed in the hope that it will be useful, 
-## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
-## GNU General Public License for more details.
-## You should have received a copy of the GNU General Public License
-## along with Lino; if not, see <http://www.gnu.org/licenses/>.
+# Copyright 2011-2014 Luc Saffre
+# License: BSD (see file COPYING for details)
 
 import logging
 logger = logging.getLogger(__name__)
 
+import django
+
 from django.conf import settings
 from django.db.models.signals import class_prepared
 from django.db.models.fields import FieldDoesNotExist
@@ -25,16 +17,56 @@ from lino.core.signals import pre_analyze
 PENDING_INJECTS = dict()
 PREPARED_MODELS = dict()
 
+
+def clear_field_cache(self):
+    """
+    Modified copy of the Django 1.6 add_field method of django.db.models.options.Options
+    Here we don't add a field, we just delete the cache variables.
+    """
+    if hasattr(self, '_m2m_cache'):
+        del self._m2m_cache
+    if hasattr(self, '_field_cache'):
+        del self._field_cache
+        del self._field_name_cache
+        try:
+            del self.fields
+        except AttributeError:
+            pass
+        try:
+            del self.concrete_fields
+        except AttributeError:
+            pass
+        try:
+            del self.local_concrete_fields
+        except AttributeError:
+            pass
+
+    if hasattr(self, '_name_map'):
+        del self._name_map
+
+
 def fix_field_cache(model):
     """
+    _fill_fields_cache
     Remove duplicate entries in `_field_cache` to fix Django issue #10808
     """
+
     cache = []
     field_names = set()
-    for f,m in model._meta._field_cache:
-        if f.attname not in field_names:
+    duplicates = []
+    for f, m in model._meta._field_cache:
+        if f.attname in field_names:
+            duplicates.append(f)
+        else:
             field_names.add(f.attname)
-            cache.append( (f,m) )
+            cache.append((f, m))
+
+    if len(duplicates) == 0:
+        return
+        #~ raise Exception("20131110 %r" % (model._meta._field_cache,))
+    #~ if django.get_version().startswith('1.6'):
+        #~ return
+
     model._meta._field_cache = tuple(cache)
     model._meta._field_name_cache = [x for x, _ in cache]
     #~ if model.__name__ in ('Company','Partner'):
@@ -42,7 +74,7 @@ def fix_field_cache(model):
 
 
 @receiver(class_prepared)
-def on_class_prepared(sender,**kw):
+def on_class_prepared(sender, **kw):
     """
     This is Lino's general `class_prepared` handler.
     It does two things:
@@ -55,47 +87,66 @@ def on_class_prepared(sender,**kw):
       (A better solution would be of course to not collect them.)
     """
     #~ if sender.__name__ in ('Company','Partner'):
-        #~ print("20130212 on_class_prepared",sender)
+    #~ print("20131110 on_class_prepared",sender)
     model = sender
+    #~ if model._meta.abstract :
+        #~ """
+        #~ 20131025 :
+        #~ """
+        #~ return
     #~ return
     #~ if model is None:
-        #~ return 
+        #~ return
     k = model._meta.app_label + '.' + model.__name__
     PREPARED_MODELS[k] = model
     #~ logger.info("20120627 on_class_prepared %r = %r",k,model)
-    todos = PENDING_INJECTS.pop(k,None)
+    todos = PENDING_INJECTS.pop(k, None)
     if todos is not None:
-        for f in todos:
-            f(model)
+        for func, caller in todos:
+            func(model)
         #~ for k,v in injects.items():
             #~ model.add_to_class(k,v)
 
     """
     django.db.models.options
     """
-    if hasattr(model._meta,'_field_cache'):
+    if hasattr(model._meta, '_field_cache'):
         fix_field_cache(model)
     #~ else:
-        #~ logger.info("20120905 Could not fix Django issue #10808 for %s",model)
+        # ~ logger.info("20131110 Could not fix Django issue #10808 for %s",model)
 
 
+import inspect
+
+
+def fmt(func_caller):
+    f, caller = func_caller
+    #~ ln = inspect.getsourcelines(f)[1]
+    #~ return "%s in %s:%d" % (f.__name__,inspect.getsourcefile(f),ln)
+    #~ return "%s in %s:%d" % (f.__name__,caller.filename,caller.line_no)
+    #~ return "%s in %s" % (f.__name__,caller)
+    return "called from %s" % caller
 
 
 @receiver(pre_analyze)
-def check_pending_injects(sender,models_list=None,**kw): # def check_pending_injects(signal,sender,models_list=None,**kw): 
+# def check_pending_injects(signal,sender,models_list=None,**kw):
+def check_pending_injects(sender, models_list=None, **kw):
     # called from kernel.analyze_models()
     site = sender
     #~ logger.info("20130212 check_pending_injects()...")
     if PENDING_INJECTS:
         msg = ''
-        for spec,funcs in PENDING_INJECTS.items():
-            msg += spec + ': ' 
+        for spec, funcs in PENDING_INJECTS.items():
+            msg += spec + ': '
+            msg += ', '.join([fmt(f) for f in funcs])
             #~ msg += '\n'.join([str(dir(func)) for func in funcs])
             #~ msg += '\n'.join([str(func.func_code.co_consts) for func in funcs])
-            msg += str(funcs)
+            #~ msg += str(funcs)
         raise Exception("Oops, there are pending injects: %s" % msg)
         #~ logger.warning("pending injects: %s", msg)
 
+    #~ logger.info("20131110 no pending injects")
+
     """
     20130106
     now we loop a last time over each model and fill it's _meta._field_cache
@@ -104,33 +155,45 @@ def check_pending_injects(sender,models_list=None,**kw): # def check_pending_inj
     """
     #~ for model in models.get_models():
     for model in models_list:
+        clear_field_cache(model._meta)
         model._meta._fill_fields_cache()
         fix_field_cache(model)
 
 #~ from lino.core.dbutils import is_installed_model_spec
 
-def do_when_prepared(todo,*model_specs):
+
+def do_when_prepared(todo, *model_specs):
     """
-    Execute the specified function `todo` on all specified models, 
-    but only when they are prepared. 
-    If a specified model hasn't yet been prepared, 
-    adds the call to a queue and execute it later.
+    Execute the specified function `todo` on all specified models
+    as soon as they are prepared.
+    If a specified model hasn't yet been prepared,
+    add the call to a queue and execute it later.
     """
+    #~ caller = inspect.stack()[2]
+    caller = inspect.getouterframes(inspect.currentframe())[2]
+    #~ print 20131111, caller
+    caller = "%s:%d" % (caller[1], caller[2])
+
+    #~ caller = inspect.getframeinfo(caller)
+    #~ caller = inspect.getframeinfo(inspect.currentframe().f_back)[2]
+    #~ caller = inspect.getframeinfo(caller.f_back)[2]
+
     for model_spec in model_specs:
         if model_spec is None:
-            continue # e.g. inject_field during autodoc when user_model is None
-            
-        if isinstance(model_spec,basestring):
-            if not settings.SITE.is_installed_model_spec(model_spec):
-                continue
+            # e.g. inject_field during autodoc when user_model is None
+            continue
+
+        if isinstance(model_spec, basestring):
+            #~ if not settings.SITE.is_installed_model_spec(model_spec):
+                #~ continue
             k = model_spec
-            model = PREPARED_MODELS.get(k,None)
-            if model is None: 
-                injects = PENDING_INJECTS.setdefault(k,[])
-                injects.append(todo)
+            model = PREPARED_MODELS.get(k, None)
+            if model is None:
+                injects = PENDING_INJECTS.setdefault(k, [])
+                injects.append((todo, caller))
                 #~ d[name] = field
                 #~ if model_spec == "system.SiteConfig":
-                    #~ logger.info("20130228 Defer %s for %s", todo, model_spec)
+                #~ logger.info("20131110 Defer %s for %s", todo, model_spec)
                 continue
         else:
             model = model_spec
@@ -142,26 +205,39 @@ def do_when_prepared(todo,*model_specs):
 
 def when_prepared(*model_specs):
     """
-    Decorator to declare a function which will automatically run when 
+    Decorator to declare a function which will automatically run when
     the specified models has been prepared.
-    If the model has already been prepared, the function is executed 
+    If the model has already been prepared, the function is executed
     immediately.
     """
     def decorator(fn):
-        return do_when_prepared(fn,*model_specs)
+        return do_when_prepared(fn, *model_specs)
     return decorator
-  
-    
 
-def inject_action(model_spec,**actions):
+
+def inject_action(model_spec, **actions):
     """
     Add the specified action(s) to the specified model.
     """
     def todo(model):
         model.define_action(**actions)
-    return do_when_prepared(todo,model_spec)    
-    
-def inject_field(model_spec,name,field,doc=None):
+    return do_when_prepared(todo, model_spec)
+
+
+def update_model(model_spec, **actions):
+    """
+    Replace the specified attributes in the specified model.q
+    """
+    def todo(model):
+        for k, v in actions.items():
+            if not hasattr(model, k):
+                raise Exception(
+                    "%s has no attribute %s to update." % (model, k))
+            setattr(model, k, v)
+    return do_when_prepared(todo, model_spec)
+
+
+def inject_field(model_spec, name, field, doc=None):
     """
     Add the given field to the given model.
     See also :doc:`/tickets/49`.
@@ -174,26 +250,23 @@ def inject_field(model_spec,name,field,doc=None):
     """
     if doc:
         field.__doc__ = doc
+
     def todo(model):
-        model.add_to_class(name,field)
+        #~ logger.info("20131110 gonna inject_field %s %s",model.__name__,name)
+        model.add_to_class(name, field)
         model._meta._fill_fields_cache()
         fix_field_cache(model)
 
-    return do_when_prepared(todo,model_spec)    
-    
-    
+    return do_when_prepared(todo, model_spec)
 
-def update_field(model_spec,name,**kw):
-    """
-    Update some attribute of the specified existing field.
-    For example 
-    :class:`PersonMixin <lino.modlib.contacts.models.PersonMixin>` 
-    defines a field `first_name` which may not be blank.
-    If you inherit from 
-    :class:`PersonMixin <lino.modlib.contacts.models.PersonMixin>`
-    but want `first_name` to be optional::
+
+def update_field(model_spec, name, **kw):
+    """Update some attribute of the specified existing field.  For
+    example :class:`Human <lino.mixins.human.Human>` defines a field
+    `first_name` which may not be blank.  If you inherit from this
+    mixin but want `first_name` to be optional::
     
-      class MyPerson(contacts.PersonMixin):
+      class MyPerson(mixins.Human):
         ...
       dd.update_field(MyPerson,'first_name',blank=True)
       
@@ -201,26 +274,27 @@ def update_field(model_spec,name,**kw):
     as done in  :mod:`lino.modlib.outbox.models`::
     
       dd.update_field(Mail,'user',verbose_name=_("Sender"))
-    
+
     """
     def todo(model):
         try:
             fld = model._meta.get_field(name)
             #~ fld = model._meta.get_field_by_name(name)[0]
         except FieldDoesNotExist:
-            logger.warning("Cannot update unresolved field %s.%s", model,name)
+            logger.warning("Cannot update unresolved field %s.%s", model, name)
             return
         if fld.model != model:
-            raise Exception('20120715 update_field(%s.%s) : %s' % (model,fld,fld.model))
+            raise Exception('20120715 update_field(%s.%s) : %s' %
+                            (model, fld, fld.model))
             #~ logger.warning('20120715 update_field(%s.%s) : %s',model,fld,fld.model)
-        for k,v in kw.items():
-            setattr(fld,k,v)
+        for k, v in kw.items():
+            setattr(fld, k, v)
         #~ if model.__name__ == "SiteConfig":
             #~ logger.info("20130228 updated field %s in %s",name,model)
-    return do_when_prepared(todo,model_spec)    
-        
+    return do_when_prepared(todo, model_spec)
 
-def inject_quick_add_buttons(model,name,target):
+
+def inject_quick_add_buttons(model, name, target):
     """
     Injects a virtual display field `name` into the specified `model`.
     This field will show up to three buttons
@@ -228,9 +302,10 @@ def inject_quick_add_buttons(model,name,target):
     `target` is the table that will run these actions.
     It must be a slave of `model`.
     """
-    def fn(self,ar):
+
+    def fn(self, ar):
         return ar.renderer.quick_add_buttons(
-          ar.spawn(target,master_instance=self))
-    inject_field(model,name,
-        fields.VirtualField(fields.DisplayField(
-            target.model._meta.verbose_name_plural),fn))
+            ar.spawn(target, master_instance=self))
+    inject_field(model, name,
+                 fields.VirtualField(fields.DisplayField(
+                     target.model._meta.verbose_name_plural), fn))
