@@ -1,38 +1,26 @@
 # -*- coding: UTF-8 -*-
-## Copyright 2010-2013 Luc Saffre
-## This file is part of the Lino project.
-## Lino is free software; you can redistribute it and/or modify 
-## it under the terms of the GNU General Public License as published by
-## the Free Software Foundation; either version 3 of the License, or
-## (at your option) any later version.
-## Lino is distributed in the hope that it will be useful, 
-## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
-## GNU General Public License for more details.
-## You should have received a copy of the GNU General Public License
-## along with Lino; if not, see <http://www.gnu.org/licenses/>.
-"""
-Loads all Belgian cities into the database.
-Supposes that :mod:`few_cities <lino.modlib.countries.fixtures.few_cities>` 
-has also been loaded and thus does not load the cities defined there.
+# Copyright 2010-2014 Luc Saffre
+# License: BSD (see file COPYING for details)
+
+"""Loads all Belgian cities into the database.  Supposes that
+:mod:`few_cities <lino.modlib.countries.fixtures.few_cities>` has also
+been loaded and thus does not load the cities defined there.
 
 This list of Belgian cities is in no way authoritative, but "works for me".
 
-The result is available for public access under 
+The result is available for public access under
 http://belref.lino-framework.org
 
-Original sources 
+Original sources:
 
 - http://www.charline.be/info/codepost/cpost.htm
 - https://fr.wikibooks.org/wiki/N%C3%A9erlandais_pour_traducteurs_:_les_villes_belges
 
 """
 
-from north.dbutils import babel_values
-
 from lino.core.dbutils import resolve_model
 from lino.utils.instantiator import Instantiator
-from lino import dd
+from lino import dd, rt
 
 
 # german names are my spontaneous guessings...
@@ -48,6 +36,8 @@ belgian_cities_nl_fr = u"""
 8600 |Diksmuide           |Dixmude              |Diksmuide
 7850 |Edingen             |Enghien              |Enghien
 8587 |Spiere-Helkijn      |Espierres-Helchin    |Spiere-Helkijn
+8587 |Spiere              |Espierres            |Spiere
+8587 |Helkijn             |Helchin              |Helkijn
 3790 |Voeren              |Fourons              |Fourons 
 8630 |Veurne              |Furnes               |Veurne
 1570 |Galmaarden          |Gammerages           |Galmaarden
@@ -79,8 +69,8 @@ belgian_cities_nl_fr = u"""
 3700 |Tongeren            |Tongres              |Tongeren
 1800 |Vilvoorde           |Vilvorde             |Vilvoorde
 8900 |Ieper               |Ypres                |Ypres
-"""                                           
-                                              
+"""
+
 belgian_cities = u"""
 9420 Aaigem 
 8511 Aalbeke 
@@ -727,7 +717,7 @@ belgian_cities = u"""
 5363 Emptinne 
 9700 Ename 
 3800 Engelmanshoven 
-7850 Enghien 
+# 7850 Enghien 
 4480 Engis 
 1350 Enines 
 4800 Ensival 
@@ -756,8 +746,8 @@ belgian_cities = u"""
 7760 Escanaffles 
 8600 Esen 
 4130 Esneux 
-8587 Espierres 
-8587 Espierres-Helchin 
+# 8587 Espierres 
+# 8587 Espierres-Helchin 
 7502 Esplechin 
 7743 Esquelmes 
 2910 Essen 
@@ -1088,12 +1078,12 @@ belgian_cities = u"""
 2220 Heist-Op-Den-Berg 
 1790 Hekelgem 
 3870 Heks 
-8587 Helchin 
+# 8587 Helchin 
 3530 Helchteren 
 9450 Heldergem 
 1357 Hélécine 
 3440 Helen-Bos 
-8587 Helkijn 
+# 8587 Helkijn 
 7830 Hellebecq 
 9571 Hemelveerdegem 
 2620 Hemiksem 
@@ -1745,7 +1735,7 @@ belgian_cities = u"""
 7140 Morlanwelz-Mariemont 
 6997 Mormont 
 5190 Mornimont 
-4670 Mortier 
+# 4670 Mortier 
 4607 Mortroux 
 2640 Mortsel 
 5620 Morville 
@@ -2389,7 +2379,7 @@ belgian_cities = u"""
 4900 Spa 
 3510 Spalbeek 
 7032 Spiennes 
-8587 Spiere 
+# 8587 Spiere 
 5530 Spontin 
 4140 Sprimont 
 5190 Spy 
@@ -2517,7 +2507,7 @@ belgian_cities = u"""
 6890 Transinne 
 6183 Trazegnies 
 5670 Treignes 
-4670 Trembleur 
+# 4670 Trembleur 
 3120 Tremelo 
 7100 Trivières 
 4280 Trognée 
@@ -2889,20 +2879,21 @@ fr = u"""
 59651 VILLENEUVE D'ASCQ 
 """
 
+
 def objects():
     countries = dd.resolve_app('countries')
-    city = Instantiator(countries.City,"zip_code name",
-        country='BE',
-        type=countries.CityTypes.city).build
+    city = Instantiator(countries.Place, "zip_code name",
+                        country='BE',
+                        type=countries.PlaceTypes.city).build
     for ln in belgian_cities.splitlines():
         ln = ln.strip()
         if ln and ln[0] != '#':
-            args = ln.split(None,1)
+            args = ln.split(None, 1)
             o = city(*args)
             # print "%r %r" % (o.zip_code, o.name)
             yield o
             #~ print __name__, "20121114"
-            #~ return 
+            #~ return
     for ln in belgian_cities_nl_fr.splitlines():
         ln = ln.strip()
         if ln and ln[0] != '#':
@@ -2910,5 +2901,6 @@ def objects():
             if len(args) != 4:
                 raise Exception("Invalid format : \n%s" % ln)
             args = [x.strip() for x in args]
-            o = city(zip_code=args[0],**babel_values('name',nl=args[1],fr=args[2],de=args[3],en=args[3]))
+            o = city(zip_code=args[0], **dd.babel_values('name',
+                     nl=args[1], fr=args[2], de=args[3], en=args[3]))
             yield o
