@@ -2,6 +2,7 @@
 import os
 import sys
 if __name__ == "__main__":
-    os.environ['DJANGO_SETTINGS_MODULE'] = 'lino.projects.babel_tutorial.settings'
+    os.environ[
+        'DJANGO_SETTINGS_MODULE'] = 'lino.projects.babel_tutorial.settings'
     from django.core.management import execute_from_command_line
     execute_from_command_line(sys.argv)
