@@ -1,5 +1,4 @@
 from lino.projects.std.settings import *
 #~ class Lino(Lino):
     #~ user_model = None
-SITE = Site(globals(),['lino','lino.test_apps.mti'])
-
+SITE = Site(globals(), ['lino.test_apps.mti'])
